﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;

namespace AdornmentLayerCSharp
{
    public class CustomAdornmentLayer : ThinkGeo.MapSuite.Core.AdornmentLayer
    {
        float x;
        float y;
        Dictionary<string, Style> legendItems;

        public CustomAdornmentLayer(float x, float y, Dictionary<string, Style> legendItems)
        {
            this.x = x;
            this.y = y;
            this.legendItems = legendItems;
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            canvas.DrawTextWithScreenCoordinate("My Legend", new GeoFont("Arial", 14), new GeoSolidBrush(GeoColor.StandardColors.Blue), x, y, DrawingLevel.LevelFour);
            y = y + 30;
            foreach (KeyValuePair<string, Style> legendItem in legendItems)
            {
                if (legendItem.Value is PointStyle)
                {
                    DrawPoint(canvas, x, y, legendItem);
                }
                else if (legendItem.Value is AreaStyle)
                {
                    DrawRectangle(canvas, x, y, 8, legendItem);
                }
                y = y + 15;
            }
        }

        private void DrawPoint(GeoCanvas canvas, float pX, float pY, KeyValuePair<string, Style> legendItem)
        {
            PointShape worldPointShape = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, pX, pY, canvas.Width, canvas.Height);
            legendItem.Value.Draw(new PointShape[] { worldPointShape }, canvas, new Collection<SimpleCandidate>(), new Collection<SimpleCandidate>());
            DrawText(canvas, pX, pY, legendItem.Key);
        }

        private void DrawRectangle(GeoCanvas canvas, float pX, float pY, float size, KeyValuePair<string, Style> legendItem)
        {
            PointShape worldPointShape = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, pX, pY, canvas.Width, canvas.Height);
            canvas.DrawArea(GetPointArrayOfARectangle(pX, pY, size), new GeoPen(GeoColor.StandardColors.Black, 1), ((AreaStyle)legendItem.Value).FillSolidBrush, DrawingLevel.LevelFour, 0, 0, PenBrushDrawingOrder.BrushFirst);
            legendItem.Value.Draw(new PointShape[] { worldPointShape }, canvas, new Collection<SimpleCandidate>(), new Collection<SimpleCandidate>());
            DrawText(canvas, pX, pY, legendItem.Key);
        }

        private IEnumerable<ScreenPointF[]> GetPointArrayOfARectangle(float pX, float pY, float size)
        {
            ScreenPointF p1 = new ScreenPointF(pX - size / 2, pY - size / 2);
            ScreenPointF p2 = new ScreenPointF(pX + size / 2, pY - size / 2);
            ScreenPointF p3 = new ScreenPointF(pX + size / 2, pY + size / 2);
            ScreenPointF p4 = new ScreenPointF(pX - size / 2, pY + size / 2);

            ScreenPointF[] pointfs = new ScreenPointF[] { p1, p2, p3, p4 };
            List<ScreenPointF[]> result = new List<ScreenPointF[]>();
            result.Add(pointfs);
            return result;
        }

        private void DrawText(GeoCanvas canvas, float pX, float pY, string pText)
        {

            DrawingFontStyles fontStyle = DrawingFontStyles.Regular;
            GeoFont Font = new GeoFont("Arial", 10, fontStyle);
            DrawingRectangleF text = canvas.MeasureText(pText, Font);

            // add text 
            pX = pX + 15 + (text.Width / 2);
            canvas.DrawTextWithScreenCoordinate(pText, Font, new GeoSolidBrush(GeoColor.StandardColors.Blue), pX, pY, DrawingLevel.LevelThree);
        }
    } 
}
